@@ -1,4 +1,3 @@
-# $Id: stockholm.pm 15093 2008-12-05 02:46:40Z cjfields $
 #
 # BioPerl module for Bio::AlignIO::stockholm
 #
@@ -107,15 +106,99 @@ the alignment annotation data.
     ----------------------------------------------------------------------
   * RN is generated based on the number of Bio::Annotation::Reference objects
 
+=head2 Custom annotation
+
+Some users may want to add custom annotation beyond those mapped above.
+Currently there are two methods to do so; however, the methods used for adding
+such annotation may change in the future, particularly if alignment Writer
+classes are introduced. In particular, do not rely on changing the global
+variables @WRITEORDER or %WRITEMAP as these may be made private at some point.
+
+1) Use (and abuse) the 'custom' tag.  The tagname for the object can differ
+from the tagname used to store the object in the AnnotationCollection.  
+
+    # AnnotationCollection from the SimpleAlign object
+    my $coll = $aln->annotation; 
+    my $factory = Bio::Annotation::AnnotationFactory->new(-type => 
+        Bio::Annotation::SimpleValue');
+    my $rfann = $factory->create_object(-value => $str, 
+                                        -tagname => 'mytag');
+    $coll->add_Annotation('custom', $rfann);
+    $rfann = $factory->create_object(-value => 'foo',
+                                    -tagname => 'bar');
+    $coll->add_Annotation('custom', $rfann);
+
+OUTPUT:
+
+    # STOCKHOLM 1.0
+    
+    #=GF ID myID12345
+    #=GF mytag katnayygqelggvnhdyddlakfyfgaglealdffnnkeaaakiinwvaEDTTRGKIQDLV??
+    #=GF mytag TPtd~????LDPETQALLV???????????????????????NAIYFKGRWE?????????~??
+    #=GF mytag ??HEF?A?EMDTKPY??DFQH?TNen?????GRI??????V???KVAM??MF?????????N??
+    #=GF mytag ???DD?VFGYAEL????DE???????L??D??????A??TALELAY??????????????????
+    #=GF mytag ?????????????KG??????Sa???TSMLILLP???????????????D??????????????
+    #=GF mytag ???????????EGTr?????AGLGKLLQ??QL????????SREef??DLNK??L???AH????R
+    #=GF mytag ????????????L????????????????????????????????????????R?????????R
+    #=GF mytag ??QQ???????V???????AVRLPKFSFefefdlkeplknlgmhqafdpnsdvfklmdqavlvi
+    #=GF mytag gdlqhayafkvd????????????????????????????????????????????????????
+    #=GF mytag ????????????????????????????????????????????????????????????????
+    #=GF mytag ????????????????????????????????????????????????????????????????
+    #=GF mytag ????????????????????????????????????????????????????????????????
+    #=GF mytag ?????????????INVDEAG?TEAAAATAAKFVPLSLppkt??????????????????PIEFV
+    #=GF mytag ADRPFAFAIR??????E?PAT?G????SILFIGHVEDPTP?msv?
+    #=GF bar foo
+    ...
+
+2) Modify the global @WRITEORDER and %WRITEMAP.
+
+    # AnnotationCollection from the SimpleAlign object
+    my $coll = $aln->annotation;
+    
+    # add to WRITEORDER
+    my @order = @Bio::AlignIO::stockholm::WRITEORDER;
+    push @order, 'my_stuff';
+    @Bio::AlignIO::stockholm::WRITEORDER = @order;
+    
+    # make sure new tag maps to something
+    $Bio::AlignIO::stockholm::WRITEMAP{my_stuff} = 'Hobbit/SimpleValue';
+
+    my $rfann = $factory->create_object(-value => 'Frodo',
+                                        -tagname => 'Hobbit');
+    $coll->add_Annotation('my_stuff', $rfann);
+    $rfann = $factory->create_object(-value => 'Bilbo',
+                                     -tagname => 'Hobbit');
+    $coll->add_Annotation('my_stuff', $rfann);
+
+OUTPUT:
+
+    # STOCKHOLM 1.0
+    
+    #=GF ID myID12345
+    #=GF Hobbit Frodo
+    #=GF Hobbit Bilbo
+    ....
+
 =head1 FEEDBACK
 
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
+
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution.  Bug reports can be submitted via the
 web:
 
-  http://bugzilla.open-bio.org/
+  https://redmine.open-bio.org/projects/bioperl/
 
 =head1 AUTHORS - Chris Fields, Peter Schattner
 
@@ -140,12 +223,11 @@ use strict;
 
 use Bio::Seq::Meta;
 use Bio::AlignIO::Handler::GenericAlignHandler;
-use Data::Dumper;
 use Text::Wrap qw(wrap);
 
 use base qw(Bio::AlignIO);
 
-our $STKVERSION = 'STOCKHOLM 1.0';
+my $STKVERSION = 'STOCKHOLM 1.0';
 
 # This maps the two-letter annotation key to a Annotation/parameter/tagname
 # combination.  Some data is stored using get/set methods ('Methods')  The rest 
@@ -184,8 +266,6 @@ my %MAPPING = (
     'SEQUENCE' => 'SEQUENCE'
 );
 
-my %CONCATENATE = map {$_ => 1} qw(DE AU DC RC RT RA RL CC);
-
 # this is the order that annotations are written
 our @WRITEORDER = qw(accession
   id
@@ -216,38 +296,39 @@ our @WRITEORDER = qw(accession
 # are mapped b/c of more complex annotation types.
 
 our %WRITEMAP = (
-            'accession'             =>  'AC/Method',
-            'id'                    =>  'ID/Method',
-            'description'           =>  'DE/Method',
-            'record_authors'        =>  'AU/SimpleValue',
-            'seed_source'           =>  'SE/SimpleValue',
-            'build_command'         =>  'BM/SimpleValue',
-            'gathering_threshold'   =>  'GA/SimpleValue',
-            'noise_cutoff'          =>  'NC/SimpleValue',
-            'trusted_cutoff'        =>  'TC/SimpleValue',
-            'entry_type'            =>  'TP/SimpleValue',
-            'num_sequences'         =>  'SQ/SimpleValue',
-            'previous_ids'          =>  'PI/SimpleValue',
-            'database_comment'      =>  'DC/SimpleValue',
-            'dblink'                =>  'DR/DBLink',
-            'reference'             =>  'RX/Reference',
-            'ref_number'            =>  'RN/number',
-            'ref_comment'           =>  'RC/comment',
-            'ref_pubmed'            =>  'RM/pubmed',
-            'ref_title'             =>  'RT/title',
-            'ref_authors'           =>  'RA/authors',
-            'ref_location'          =>  'RL/location',
-            'alignment_comment'     =>  'CC/Comment',
-            'seq_annotation'        =>  'DR/Collection',
-            #Pfam-specific 
-            'build_method'          =>  'AM/SimpleValue',
-            'pfam_family_accession' =>  'NE/SimpleValue',
-            'seq_start_stop'        =>  'NL/SimpleValue',
-            # Rfam-specific GF lines
-            'sec_structure_source'  =>  'SS/SimpleValue',
-            # custom
-            'custom'                =>  'XX/SimpleValue'
-            );
+    'accession'             =>  'AC/Method',
+    'id'                    =>  'ID/Method',
+    'description'           =>  'DE/Method',
+    'record_authors'        =>  'AU/SimpleValue',
+    'seed_source'           =>  'SE/SimpleValue',
+    'build_command'         =>  'BM/SimpleValue',
+    'gathering_threshold'   =>  'GA/SimpleValue',
+    'noise_cutoff'          =>  'NC/SimpleValue',
+    'trusted_cutoff'        =>  'TC/SimpleValue',
+    'entry_type'            =>  'TP/SimpleValue',
+    'num_sequences'         =>  'SQ/SimpleValue',
+    'previous_ids'          =>  'PI/SimpleValue',
+    'database_comment'      =>  'DC/SimpleValue',
+    'dblink'                =>  'DR/DBLink',
+    'reference'             =>  'RX/Reference',
+    'ref_number'            =>  'RN/number',
+    'ref_comment'           =>  'RC/comment',
+    'ref_pubmed'            =>  'RM/pubmed',
+    'ref_title'             =>  'RT/title',
+    'ref_authors'           =>  'RA/authors',
+    'ref_location'          =>  'RL/location',
+    'alignment_comment'     =>  'CC/Comment',
+    'seq_annotation'        =>  'DR/Collection',
+    #Pfam-specific 
+    'build_method'          =>  'AM/SimpleValue',
+    'pfam_family_accession' =>  'NE/SimpleValue',
+    'seq_start_stop'        =>  'NL/SimpleValue',
+    # Rfam-specific GF lines
+    'sec_structure_source'  =>  'SS/SimpleValue',
+    # custom; this is used to carry over anything from the input alignment
+    # not mapped to LocatableSeqs or SimpleAlign in a meaningful way
+    'custom'                =>  'XX/SimpleValue'
+);
 
 # This maps the tagname back to a tagname-annotation value combination.
 # Some data is stored using get/set methods ('Methods'), others
@@ -256,9 +337,9 @@ our %WRITEMAP = (
 =head2 new
 
  Title   : new
- Usage   : my $alignio = Bio::AlignIO->new(-format => 'phylip'
+ Usage   : my $alignio = Bio::AlignIO->new(-format => 'stockholm'
 					  -file   => '>file');
- Function: Initialize a new L<Bio::AlignIO::phylip> reader or writer
+ Function: Initialize a new L<Bio::AlignIO::stockholm> reader or writer
  Returns : L<Bio::AlignIO> object
  Args    : -line_length :  length of the line for the alignment block
            -alphabet    :  symbol alphabet to set the sequences to.  If not set,
@@ -300,8 +381,8 @@ sub next_aln {
     
     my $handler = $self->alignhandler;
     # advance to alignment header
-    while( defined(my $line = $self->_readline) ) {
-        if ($line =~ m{^#\s*STOCKHOLM\s+}xmso) {
+    while( defined(my $line = $self->_readline) ) {        
+        if ($line =~ m{^\#\s*STOCKHOLM\s+}xmso) {
             last;
         }
     }
@@ -338,7 +419,7 @@ sub next_aln {
             }
             $align = ($primary_tag eq 'GF' || $primary_tag eq 'GR') ? 1 : 0;
         }
-        elsif ($line =~ m{^([^\#]\S+)\s+([A-Za-z.\-\*]+)\s*}) {
+        elsif ($line =~ m{^([^\#]\S+)\s+([^\s]+)\s*}) {
             $self->{block_line}++;
             ($feat, $nse, $data) = ('SEQUENCE', $1, $2);
         }
@@ -401,7 +482,7 @@ sub next_aln {
         $last_feat = $feat;
     }
     
-    my $aln = $handler->build_alignment;    
+    my $aln = $handler->build_alignment;
     $handler->reset_parameters;
     return $aln;
 }
@@ -484,22 +565,26 @@ sub write_aln {
             }
             elsif ($tag eq 'XX') { # custom
                 my $newtag = $ann->tagname;
-                $alntag = sprintf('%-10s',$aln_ann.$newtag);
+                my $tmp = $aln_ann.$newtag;
+                $alntag = sprintf('%-*s',length($tmp) + 1, $tmp);
                 $data = $ann->display_text;
             }
             elsif ($tag eq 'SQ') {
                 # use the actual number, not the stored Annotation data
-                $alntag = sprintf('%-10s',$aln_ann.$tag);
-                $data = $aln->no_sequences;
+                my $tmp = $aln_ann.$tag;
+                $alntag = sprintf('%-*s',length($tmp) + 1, $tmp);
+                $data = $aln->num_sequences;
             }
             elsif ($tag eq 'DR') {
-                $alntag = sprintf('%-10s',$aln_ann.$tag);
+                my $tmp = $aln_ann.$tag;
+                $alntag = sprintf('%-*s',length($tmp) + 1, $tmp);
                 my $db = uc $ann->database;
                 my $cb = exists $LINK_CB{$db} ? $LINK_CB{$db} : $LINK_CB{_DEFAULT_};
                 $data = $ann->display_text($cb);
             }
             else {
-                $alntag = sprintf('%-10s',$aln_ann.$tag);
+                my $tmp = $aln_ann.$tag;
+                $alntag = sprintf('%-*s',length($tmp) + 1, $tmp);
                 $data = ref $ann ? $ann->display_text : $ann;
             }
             next unless $data;
@@ -512,8 +597,9 @@ sub write_aln {
     my $tag = 'AC';
     for my $seq ($aln->each_seq) {
         if (my $acc = $seq->accession_number) {
-        my $text = sprintf("%-4s%-22s%-3s%s\n",$seq_ann, $seq->get_nse, $tag, $acc);
-        $self->_print($text) || return 0;
+	    my $text = sprintf("%-4s%-22s%-3s%s\n",$seq_ann, 
+			       $aln->displayname($seq->get_nse), $tag, $acc);
+	    $self->_print($text) || return 0;
         }
     }
     
@@ -525,7 +611,7 @@ sub write_aln {
                 my $db = uc $link->database;
                 my $cb = exists $LINK_CB{$db} ? $LINK_CB{$db} : $LINK_CB{_DEFAULT_};
                 my $text = sprintf("%-4s%-22s%-3s%s\n",$seq_ann,
-                                   $sf->entire_seq->get_nse,
+                                   $aln->displayname($sf->entire_seq->get_nse),
                                    $tag,
                                    $link->display_text($cb));
                 $self->_print($text) || return 0;
@@ -579,25 +665,6 @@ sub line_length {
     return $self->{'_line_length'};
 }
 
-=head2 alphabet
-
- Title   : alphabet
- Usage   : $obj->alphabet('dna')
- Function: Set the sequence data alphabet
- Returns : sequence data type
- Args    : newvalue (optional)
-
-=cut
-
-sub alphabet {
-    my ( $self, $value ) = @_;
-    if ( defined $value ) {
-        $self->throw("Invalid alphabet $value") unless $value eq 'rna' || $value eq 'protein' || $value eq 'dna';
-        $self->{'_alphabet'} = $value;
-    }
-    return $self->{'_alphabet'};
-};
-
 =head2 spaces
 
  Title   : spaces
@@ -635,20 +702,6 @@ sub alignhandler {
     return $self->{'_alignhandler'};
 }
 
-=head2 alignwriter
-
- Title   : alignwriter
- Usage   : $stream->alignwriter($writer)
- Function: Get/Set the writer object
- Returns : 
- Args    : 
-
-=cut
-
-sub alignwriter {
-    shift->throw_not_implemented;
-}
-
 ############# PRIVATE INIT/HANDLER METHODS #############
 
 sub _print_seqs {
@@ -663,7 +716,7 @@ sub _print_seqs {
 
     for $seq ( $aln->each_seq() ) {
         my ($s, $e, $str) = ($seq->start, $seq->end, $seq->strand);
-        $namestr = $seq->get_nse();
+        $namestr = $aln->displayname($seq->get_nse());
         $self->_print(sprintf("%-*s%s\n",$maxlen+$metalen,
                               $namestr,
                               $seq->seq())) || return 0;
