@@ -1,5 +1,4 @@
 #
-# $Id: Qual.pm 15505 2009-02-05 18:22:58Z cjfields $
 #
 # BioPerl module for Bio::DB::Qual
 #
@@ -364,10 +363,6 @@ solution could be to padd them with blank spaces so that each line has the same
 number of characters (maybe this padding should be implemented in
 Bio::SeqIO::qual?).
 
-=head1 SEE ALSO
-
-L<bioperl>
-
 =head1 AUTHOR
 
 Florent E Angly E<lt>florent . angly @ gmail-dot-comE<gt>.  
