@@ -1,6 +1,5 @@
 package Bio::DB::SeqFeature::Store::FeatureFileLoader;
 
-# $Id$
 
 =head1 NAME
 
@@ -104,7 +103,7 @@ features type and name on multiple adjacent lines:
  Gene	B0511.1	Chr1:10946..11208
 
 If all the locations are on the same reference sequence, you can
-specify a default chromosome using a "reference=<seqid>":
+specify a default chromosome using a "reference=E<lt>seqidE<gt>":
 
  reference=Chr1
  Gene	B0511.1	516..619
@@ -544,9 +543,11 @@ sub handle_feature {
   my $feature = $ld->{CurrentFeature};
   
   $ld->{OldPartType} = $ld->{PartType};
-  $ld->{PartType}    = $attr->{Type}[0] if exists $attr->{Type};
-  $ld->{PartType}    = $attr->{type}[0] if exists $attr->{type};
-  $ld->{PartType}   ||= $type;
+  if (exists $attr->{Type} || exists $attr->{type})  {
+      $ld->{PartType}   = $attr->{Type}[0] || $attr->{type}[0];
+  } else {
+      $ld->{PartType}   = $type;
+  }
 
   if ($feature) {
       local $^W = 0;  # avoid uninit warning when display_name() is called
@@ -574,7 +575,8 @@ sub handle_feature {
 					    $type,   # side effect is to set CurrentFeature
 					    $strand,
 					    $attr,
-					    $reference,@{$parts[0]});
+					    $reference,
+					    @{$parts[0]});
 
   # add more segments to the current feature
   if (@parts > 1) {
@@ -623,12 +625,23 @@ sub _make_feature {
     my $self = shift;
     my ($name,$type,$strand,$attributes,$ref,$start,$end) = @_;
 
+    # some basic error checking
+     $self->throw("syntax error at line $.: '$_'")
+ 	if ($ref   && !defined $start)
+ 	or ($ref   && !defined $end)
+ 	or ($start && $start   !~  /^[-\d]+$/)
+ 	or ($end   && $end     !~  /^[-\d]+$/)
+ 	or !defined $type
+ 	or !defined $name;
+
     $strand ||= '';
 
     my @args = (-name        => $name,
-		-strand      => $strand eq '+' ? 1 
+		-strand      => $strand eq '+' ?  1 
                                :$strand eq '-' ? -1
-		               :$strand == 1   ? 1
+		               :$strand eq ''  ?  0
+                               :$strand eq '.' ?  0
+		               :$strand == 1   ?  1
 		               :$strand == -1  ? -1
                                :0,
 		-attributes  => $attributes,
@@ -821,7 +834,6 @@ use the BioPerl bug tracking system to report bugs.
 
 =head1 SEE ALSO
 
-L<bioperl>,
 L<Bio::DB::SeqFeature::Store>,
 L<Bio::DB::SeqFeature::Segment>,
 L<Bio::DB::SeqFeature::NormalizedFeature>,
