@@ -1,6 +1,5 @@
 package Bio::DB::SeqFeature::Store::GFF3Loader;
 
-# $Id: GFF3Loader.pm 16090 2009-09-15 21:57:56Z cjfields $
 
 =head1 NAME
 
@@ -128,6 +127,11 @@ pairs as described in this table:
  -ignore_seqregion  Ignore ##sequence-region directives. The default is to create a
                        feature corresponding to the directive.
 
+ -noalias_target    Don't create an Alias attribute for a target_id named in a 
+                    Target attribute. The default is to create an Alias
+                    attribute containing the target_id found in a Target 
+                    attribute.
+
 When you call new(), a connection to a Bio::DB::SeqFeature::Store
 database should already have been established and the database
 initialized (if appropriate).
@@ -145,10 +149,10 @@ normal (slow) loading.
 If you use an unnormalized feature class, such as
 Bio::SeqFeature::Generic, then the loader needs to create a temporary
 database in which to cache features until all their parts and subparts
-have been seen. This temporary databases uses the "berkeleydb" adaptor. The
--tmp option specifies the directory in which that database will be
-created. If not present, it defaults to the system default tmp
-directory specified by File::Spec-E<gt>tmpdir().
+have been seen. This temporary databases uses the "berkeleydb"
+adaptor. The -tmp option specifies the directory in which that
+database will be created. If not present, it defaults to the system
+default tmp directory specified by File::Spec-E<gt>tmpdir().
 
 The -chunk_size option allows you to tune the representation of
 DNA/Protein sequence in the Store database. By default, sequences are
@@ -166,6 +170,8 @@ sub new {
     my $self  = $class->SUPER::new(@_);
     my ($ignore_seqregion) = rearrange(['IGNORE_SEQREGION'],@_);
     $self->ignore_seqregion($ignore_seqregion);
+    my ($noalias_target) = rearrange(['NOALIAS_TARGET'],@_);
+    $self->noalias_target($noalias_target);
     $self;
 }
 
@@ -186,6 +192,24 @@ sub ignore_seqregion {
     $d;
 }
 
+=head2 noalias_target
+
+  $noalias_target = $loader->noalias_target([$new_flag])
+
+Get or set the noalias_target flag, which if true, will disable the creation of
+an Alias attribute for a target_id named in a Target attribute. The default is 
+to create an Alias attribute containing the target_id found in a Target 
+attribute.
+
+=cut
+
+sub noalias_target {
+    my $self = shift;
+    my $d    = $self->{noalias_target};
+    $self->{noalias_target} = shift if @_;
+    $d;
+}
+
 =head2 load
 
  Title   : load
@@ -326,6 +350,7 @@ sub finish_load { #overridden
     $self->msg(sprintf "%5.2fs\n",$self->time()-$start);
   }
   eval {$self->store->commit};
+
   # don't delete load data so that caller can ask for the loaded IDs
   # $self->delete_load_data;
 }
@@ -363,7 +388,10 @@ sub load_line { #overridden
     $load_data->{line}++;
 
     return unless $line =~ /^\S/;     # blank line
-    $load_data->{mode} = 'gff' if /\t/;  # if it has a tab in it, switch to gff mode
+
+    # if it has a tab in it or looks like a chrom.sizes file, switch to gff mode
+    $load_data->{mode} = 'gff' if $line =~ /\t/
+	or $line =~ /^\w+\s+\d+\s*$/;
 
     if ($line =~ /^\#\s?\#\s*(.+)/) {  ## meta instruction
       $load_data->{mode} = 'gff';
@@ -375,8 +403,8 @@ sub load_line { #overridden
     }
 
     elsif ($line =~ /^>\s*(\S+)/) { # FASTA lines are coming
-      $load_data->{mode} = 'fasta';
-      $self->start_or_finish_sequence($1);
+	$load_data->{mode} = 'fasta';
+	$self->start_or_finish_sequence($1);
     }
 
     elsif ($load_data->{mode} eq 'fasta') {
@@ -467,9 +495,15 @@ sub handle_feature { #overridden
   my $ld       = $self->{load_data};
 
   my $allow_whitespace = $self->allow_whitespace;
-  $gff_line    =~ s/\s+/\t/g if $allow_whitespace;
 
-  my @columns = map {$_ eq '.' ? undef : $_ } split /\t/,$gff_line;
+  # special case for a chrom.sizes-style line
+  my @columns;
+  if ($gff_line =~ /^(\w+)\s+(\d+)\s*$/) {
+      @columns = ($1,undef,'chromosome',1,$2,undef,undef,undef,"Name=$1");
+  } else {
+      $gff_line    =~ s/\s+/\t/g if $allow_whitespace;
+      @columns = map {$_ eq '.' ? undef : $_ } split /\t/,$gff_line;
+  }
 
   $self->invalid_gff($gff_line) if @columns < 4;
   $self->invalid_gff($gff_line) if @columns > 9 && $allow_whitespace;
@@ -484,8 +518,8 @@ sub handle_feature { #overridden
   my ($refname,$source,$method,$start,$end,$score,$strand,$phase,$attributes) = @columns;
   
   $self->invalid_gff($gff_line) unless defined $refname;
-  $self->invalid_gff($gff_line) unless $start eq '.' || $start =~ /^[\d.-]+$/;
-  $self->invalid_gff($gff_line) unless $end   eq '.' || $end   =~ /^[\d.-]+$/;
+  $self->invalid_gff($gff_line) unless !defined $start || $start =~ /^[\d.-]+$/;
+  $self->invalid_gff($gff_line) unless !defined $end   || $end   =~ /^[\d.-]+$/;
   $self->invalid_gff($gff_line) unless defined $method;
 
   $strand = $Strandedness{$strand||0};
@@ -525,7 +559,7 @@ sub handle_feature { #overridden
 
   # POSSIBLY A PERMANENT HACK -- TARGETS BECOME ALIASES
   # THIS IS TO ALLOW FOR TARGET-BASED LOOKUPS
-  if (exists $reserved->{Target}) {
+  if (exists $reserved->{Target} && !$self->{noalias_target}) {
     my %aliases = map {$_=>1} @{$unreserved->{Alias}};
     for my $t (@{$reserved->{Target}}) {
       (my $tc = $t) =~ s/\s+.*$//;  # get rid of coordinates
@@ -992,7 +1026,6 @@ use the BioPerl bug tracking system to report bugs.
 
 =head1 SEE ALSO
 
-L<bioperl>,
 L<Bio::DB::SeqFeature::Store>,
 L<Bio::DB::SeqFeature::Segment>,
 L<Bio::DB::SeqFeature::NormalizedFeature>,
