@@ -1,7 +1,8 @@
-# $Id: PullParserI.pm 11567 2007-07-12 10:42:06Z sendu $
 #
 # BioPerl module for Bio::PullParserI
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Sendu Bala <bix@sendu.me.uk>
 #
 # Copyright Sendu Bala
@@ -92,13 +93,24 @@ the Bioperl mailing list.  Your participation is much appreciated.
   bioperl-l@bioperl.org                  - General discussion
   http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
 
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
+
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
 of the bugs and their resolution. Bug reports can be submitted via the
 web:
 
-  http://bugzilla.open-bio.org/
+  https://redmine.open-bio.org/projects/bioperl/
 
 =head1 AUTHOR - Sendu Bala
 
@@ -474,6 +486,7 @@ sub _line_ending {
 
 sub _chunk_seek {
     my ($self, $pos) = @_;
+    $self->throw("Undefined position passed") unless defined $pos;
     return if $self->_sequential;
     
     my $fh = $self->chunk->_fh;
