@@ -1,4 +1,3 @@
-# $Id: Analysis.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module Bio::Restriction::Analysis
 #
@@ -166,7 +165,7 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution. Bug reports can be submitted via the
 web:
 
-  http://bugzilla.open-bio.org/
+  https://redmine.open-bio.org/projects/bioperl/
 
 =head1 AUTHOR
 
@@ -205,8 +204,8 @@ use Bio::Restriction::EnzymeCollection;
 use strict;
 use Data::Dumper;
 
-use vars qw ();
 use base qw(Bio::Root::Root);
+use Scalar::Util qw(blessed);
 
 =head1 new
 
@@ -366,7 +365,7 @@ sub cut {
     $self->throw("A sequence must be supplied")
         unless $self->seq;
 
-    if (uc($opt) eq "MULTIPLE") {
+    if ($opt && uc($opt) eq "MULTIPLE") {
       $self->throw("You must supply a separate enzyme collection for multiple digests") unless $ec;
       $self->_multiple_cuts($ec); # multiple digests
     } else {
@@ -521,9 +520,9 @@ sub fragment_maps {
     # for every enzyme this should cut down on the amount of
     # duplicated data we are trying to save in memory and make this
     # faster and easier for large sequences, e.g. genome analysis
-
+    
     my @cut_positions;
-    if (ref $enz eq '') {
+    if (ref $enz eq '' && exists $self->{'_cut_positions'}->{$enz}) {
         @cut_positions=@{$self->{'_cut_positions'}->{$enz}};
     } elsif ($enz->isa("Bio::Restriction::EnzymeI")) {
         @cut_positions=@{$self->{'_cut_positions'}->{$enz->name}};
@@ -532,7 +531,7 @@ sub fragment_maps {
         @cut_positions=@{$self->{'_cut_positions'}->{'multiple_digest'}};
     }
 
-    unless ($cut_positions[0]) {
+    unless (defined($cut_positions[0])) {
         # it doesn't cut
         # return the whole sequence
         # this should probably have the is_circular command
@@ -554,6 +553,7 @@ sub fragment_maps {
 
     my $start=1; my $stop; my %seq; my %stop;
     foreach $stop (@cuts) {
+        next if !$stop; # cuts at beginning of sequence
         $seq{$start}=$self->{'_seq'}->subseq($start, $stop);
         $stop{$start}=$stop;
         $start=$stop+1;
@@ -607,11 +607,11 @@ on a gel!
 You should be able to do these:
 
   # to see all the fragment sizes,
-  print join "\n", @{$re->sizes($enz)}, "\n";
+  print join "\n", $re->sizes($enz), "\n";
   # to see all the fragment sizes sorted
-  print join "\n", @{$re->sizes($enz, 0, 1)}, "\n";
+  print join "\n", $re->sizes($enz, 0, 1), "\n";
   # to see all the fragment sizes in kb sorted
-  print join "\n", @{$re->sizes($enz, 1, 1)}, "\n";
+  print join "\n", $re->sizes($enz, 1, 1), "\n";
 
 =cut
 
@@ -619,8 +619,15 @@ sub sizes {
     my ($self, $enz, $kb, $sort) = @_;
     $self->throw('no enzyme selected to get fragments for')
         unless $enz;
+    
+    if (blessed($enz)) {
+        $self->throw("Enzyme must be enzyme name or a Bio::Restriction::EnzymeI, not ".ref($enz))
+            if !$enz->isa('Bio::Restriction::EnzymeI');
+        $enz = $enz->name;
+    }
     $self->cut unless $self->{'_cut'};
     my @frag; my $lastsite=0;
+
     foreach my $site (@{$self->{'_cut_positions'}->{$enz}}) {
       $kb ? push (@frag, (int($site-($lastsite))/100)/10)
           : push (@frag, $site-($lastsite));
