@@ -1,4 +1,3 @@
-# $Id: IO.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::Root::IO
 #
@@ -92,12 +91,16 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution.  Bug reports can be submitted via the
 web:
 
-  http://bugzilla.open-bio.org/
+  https://redmine.open-bio.org/projects/bioperl/
 
 =head1 AUTHOR - Hilmar Lapp
 
 Email hlapp@gmx.net
 
+=head1 CONTRIBUTORS
+
+Mark A. Jensen ( maj -at- fortinbras -dot- us )
+
 =head1 APPENDIX
 
 The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _
@@ -109,10 +112,14 @@ The rest of the documentation details each of the object methods. Internal metho
 
 
 package Bio::Root::IO;
-use vars qw($FILESPECLOADED $FILETEMPLOADED $FILEPATHLOADED
-	    $TEMPDIR $PATHSEP $ROOTDIR $OPENFLAGS $VERBOSE $ONMAC
-            $HAS_LWP
-           );
+
+our ($FILESPECLOADED,   $FILETEMPLOADED,
+    $FILEPATHLOADED,    $TEMPDIR,
+    $PATHSEP,           $ROOTDIR,
+    $OPENFLAGS,         $VERBOSE,
+    $ONMAC,             $HAS_LWP,
+    $HAS_EOL);
+
 use strict;
 
 use Symbol;
@@ -124,7 +131,6 @@ use base qw(Bio::Root::Root);
 
 my $TEMPCOUNTER;
 my $HAS_WIN32 = 0;
-#my $HAS_LWP = 1;
 
 BEGIN {
     $TEMPCOUNTER = 0;
@@ -135,19 +141,19 @@ BEGIN {
 
     # try to load those modules that may cause trouble on some systems
     eval { 
-	require File::Path;
-	$FILEPATHLOADED = 1;
+        require File::Path;
+        $FILEPATHLOADED = 1;
     }; 
     if( $@ ) {
-	print STDERR "Cannot load File::Path: $@" if( $VERBOSE > 0 );
-	# do nothing
+        print STDERR "Cannot load File::Path: $@" if( $VERBOSE > 0 );
+        # do nothing
     }
 
     eval {
-        require LWP::Simple;
+        require LWP::UserAgent;
     };
     if( $@ ) {
-	print STDERR "Cannot load LWP::Simple: $@" if( $VERBOSE > 0 );
+        print STDERR "Cannot load LWP::UserAgent: $@" if( $VERBOSE > 0 );
         $HAS_LWP = 0;
     } else {
         $HAS_LWP = 1;
@@ -156,63 +162,63 @@ BEGIN {
     # If on Win32, attempt to find Win32 package
 
     if($^O =~ /mswin/i) {
-	eval {
-	    require Win32;
-	    $HAS_WIN32 = 1;
-	};
+    eval {
+        require Win32;
+        $HAS_WIN32 = 1;
+    };
     }
 
     # Try to provide a path separator. Why doesn't File::Spec export this,
     # or did I miss it?
     if($^O =~ /mswin/i) {
-	$PATHSEP = "\\";
+        $PATHSEP = "\\";
     } elsif($^O =~ /macos/i) {
-	$PATHSEP = ":";
+        $PATHSEP = ":";
     } else { # unix
-	$PATHSEP = "/";
+        $PATHSEP = "/";
     }
     eval {
-	require File::Spec;
-	$FILESPECLOADED = 1;
-	$TEMPDIR = File::Spec->tmpdir();
-	$ROOTDIR = File::Spec->rootdir();
-	require File::Temp; # tempfile creation
-	$FILETEMPLOADED = 1;
+        require File::Spec;
+        $FILESPECLOADED = 1;
+        $TEMPDIR = File::Spec->tmpdir();
+        $ROOTDIR = File::Spec->rootdir();
+        require File::Temp; # tempfile creation
+        $FILETEMPLOADED = 1;
     };
     if( $@ ) { 
-	if(! defined($TEMPDIR)) { # File::Spec failed
-	    # determine tempdir
-	    if (defined $ENV{'TEMPDIR'} && -d $ENV{'TEMPDIR'} ) {
-		$TEMPDIR = $ENV{'TEMPDIR'};
-	    } elsif( defined $ENV{'TMPDIR'} && -d $ENV{'TMPDIR'} ) {
-		$TEMPDIR = $ENV{'TMPDIR'};
-	    }
-	    if($^O =~ /mswin/i) {
-		$TEMPDIR = 'C:\TEMP' unless $TEMPDIR;
-		$ROOTDIR = 'C:';
-	    } elsif($^O =~ /macos/i) {
-		$TEMPDIR = "" unless $TEMPDIR; # what is a reasonable default on Macs?
-		$ROOTDIR = ""; # what is reasonable??
-	    } else { # unix
-		$TEMPDIR = "/tmp" unless $TEMPDIR;
-		$ROOTDIR = "/";
-	    }
-	    if (!( -d $TEMPDIR && -w $TEMPDIR )) {
-		$TEMPDIR = '.'; # last resort
-	    }
-	}
-	# File::Temp failed (alone, or File::Spec already failed)
-	#
-	# determine open flags for tempfile creation -- we'll have to do this
-	# ourselves
-	use Fcntl;
-	use Symbol;
-	$OPENFLAGS = O_CREAT | O_EXCL | O_RDWR;
-	for my $oflag (qw/FOLLOW BINARY LARGEFILE EXLOCK NOINHERIT TEMPORARY/){
-	    my ($bit, $func) = (0, "Fcntl::O_" . $oflag);
-	    no strict 'refs';
-	    $OPENFLAGS |= $bit if eval { $bit = &$func(); 1 };
-	}
+    if(! defined($TEMPDIR)) { # File::Spec failed
+        # determine tempdir
+        if (defined $ENV{'TEMPDIR'} && -d $ENV{'TEMPDIR'} ) {
+            $TEMPDIR = $ENV{'TEMPDIR'};
+        } elsif( defined $ENV{'TMPDIR'} && -d $ENV{'TMPDIR'} ) {
+            $TEMPDIR = $ENV{'TMPDIR'};
+        }
+        if($^O =~ /mswin/i) {
+            $TEMPDIR = 'C:\TEMP' unless $TEMPDIR;
+            $ROOTDIR = 'C:';
+        } elsif($^O =~ /macos/i) {
+            $TEMPDIR = "" unless $TEMPDIR; # what is a reasonable default on Macs?
+            $ROOTDIR = ""; # what is reasonable??
+        } else { # unix
+            $TEMPDIR = "/tmp" unless $TEMPDIR;
+            $ROOTDIR = "/";
+        }
+        if (!( -d $TEMPDIR && -w $TEMPDIR )) {
+            $TEMPDIR = '.'; # last resort
+        }
+    }
+    # File::Temp failed (alone, or File::Spec already failed)
+    #
+    # determine open flags for tempfile creation -- we'll have to do this
+    # ourselves
+    use Fcntl;
+    use Symbol;
+    $OPENFLAGS = O_CREAT | O_EXCL | O_RDWR;
+    for my $oflag (qw/FOLLOW BINARY LARGEFILE EXLOCK NOINHERIT TEMPORARY/){
+        my ($bit, $func) = (0, "Fcntl::O_" . $oflag);
+        no strict 'refs';
+        $OPENFLAGS |= $bit if eval { $bit = &$func(); 1 };
+    }
     }
     $ONMAC = "\015" eq "\n";
 }
@@ -245,12 +251,20 @@ sub new {
 
            Currently recognizes the following named parameters:
               -file     name of file to open
+              -string   a string that is to be converted to a filehandle
               -url      name of URL to open
               -input    name of file, or GLOB, or IO::Handle object
               -fh       file handle (mutually exclusive with -file)
               -flush    boolean flag to autoflush after each write
               -noclose  boolean flag, when set to true will not close a
                         filehandle (must explictly call close($io->_fh)
+              -retries  number of times to try a web fetch before failure
+                        
+              -ua_parms hashref of key => value parameters to pass 
+                        to LWP::UserAgent->new()
+                        (only meaningful with -url is set)
+                        A useful value might be, for example,
+                        { timeout => 60 } (ua default is 180 sec)
  Returns : TRUE
  Args    : named parameters
 
@@ -262,70 +276,96 @@ sub _initialize_io {
 
     $self->_register_for_cleanup(\&_io_cleanup);
 
-    my ($input, $noclose, $file, $fh, $flush, $url) = $self->_rearrange([qw(INPUT 
-							    NOCLOSE
-							    FILE FH 
-							    FLUSH URL)], @args);
+    my ($input, $noclose, $file, $fh, $string, $flush, $url,
+    $retries, $ua_parms) = 
+    $self->_rearrange([qw(INPUT NOCLOSE FILE FH STRING FLUSH URL RETRIES UA_PARMS)],
+                      @args);
 
     if($url){
-      my $trymax = 5;
-
-      if($HAS_LWP){ #use LWP::Simple::getstore()
-	require LWP::Simple;
-        #$self->warn("has lwp");
-        my $http_result;
-        my($handle,$tempfile) = $self->tempfile();
-        CORE::close($handle);
-
-        for(my $try = 1 ; $try <= $trymax ; $try++){
-          $http_result = LWP::Simple::getstore($url, $tempfile);
-          $self->warn("[$try/$trymax] tried to fetch $url, but server threw $http_result.  retrying...") if $http_result != 200;
-          last if $http_result == 200;
+        $retries ||= 5;
+    
+        if($HAS_LWP) { #use LWP::UserAgent
+            require LWP::UserAgent;
+            my $ua = LWP::UserAgent->new(%$ua_parms);
+            my $http_result;
+            my($handle,$tempfile) = $self->tempfile();
+            CORE::close($handle);
+          
+    
+            for(my $try = 1 ; $try <= $retries ; $try++){
+                $http_result = $ua->get($url, ':content_file' => $tempfile);
+                $self->warn("[$try/$retries] tried to fetch $url, but server ".
+                            "threw ". $http_result->code . ".  retrying...")
+                            if !$http_result->is_success;
+                last if $http_result->is_success;
+            }
+            $self->throw("failed to fetch $url, server threw ".
+                         $http_result->code) if !$http_result->is_success;
+    
+            $input = $tempfile;
+            $file  = $tempfile;
+        } else { #use Bio::Root::HTTPget
+            #$self->warn("no lwp");
+    
+            $fh = Bio::Root::HTTPget::getFH($url);
+        }
         }
-        $self->throw("failed to fetch $url, server threw $http_result") if $http_result != 200;
-
-        $input = $tempfile;
-        $file  = $tempfile;
-      } else { #use Bio::Root::HTTPget
-        #$self->warn("no lwp");
-
-        $fh = Bio::Root::HTTPget->getFH($url);
-      }
-    }
 
     delete $self->{'_readbuffer'};
     delete $self->{'_filehandle'};
     $self->noclose( $noclose) if defined $noclose;
     # determine whether the input is a file(name) or a stream
     if($input) {
-	if(ref(\$input) eq "SCALAR") {
-	    # we assume that a scalar is a filename
-	    if($file && ($file ne $input)) {
-		$self->throw("input file given twice: $file and $input disagree");
-	    }
-	    $file = $input;
-	} elsif(ref($input) &&
-		((ref($input) eq "GLOB") || $input->isa('IO::Handle'))) {
-	    # input is a stream
-	    $fh = $input;
-	} else {
-	    # let's be strict for now
-	    $self->throw("unable to determine type of input $input: ".
-			 "not string and not GLOB");
-	}
+        if(ref(\$input) eq "SCALAR") {
+            # we assume that a scalar is a filename
+            if($file && ($file ne $input)) {
+            $self->throw("input file given twice: $file and $input disagree");
+            }
+            $file = $input;
+        } elsif(ref($input) &&
+            ((ref($input) eq "GLOB") || $input->isa('IO::Handle'))) {
+            # input is a stream
+            $fh = $input;
+        } else {
+            # let's be strict for now
+            $self->throw("unable to determine type of input $input: ".
+                 "not string and not GLOB");
+        }
     }
+    
     if(defined($file) && defined($fh)) {
-	$self->throw("Providing both a file and a filehandle for reading - only one please!");
+        $self->throw("Providing both a file and a filehandle for reading - ".
+                     "only one please!");
     }
 
-    if(defined($file) && ($file ne '')) {
-	$fh = Symbol::gensym();
-	open ($fh,$file) ||
-	    $self->throw("Could not open $file: $!");
-	$self->file($file);
+    if ($string) {
+        if(defined($file) || defined($fh)) {
+            $self->throw("File or filehandle provided with -string,".
+                         " please unset if you are using -string as a file");
+        }
+        open($fh, "<", \$string)
     }
-    if ( defined($fh) && !(ref($fh) && ((ref($fh) eq "GLOB") || $fh->isa('IO::Handle') || $fh->isa('IO::String')))  ) {
-        $self->throw("file handle $fh doesn't appear to be a handle");
+    
+    if(defined($file) && ($file ne '')) {
+        $fh = Symbol::gensym();
+        open ($fh,$file) || $self->throw("Could not open $file: $!");
+        $self->file($file);
+    }
+
+    if (defined $fh) {
+        # check filehandle to ensure it's one of:
+        # a GLOB reference, as in: open(my $fh, "myfile");
+        # an IO::Handle or IO::String object
+        # the UNIVERSAL::can added to fix Bug2863
+        unless ( ( ref $fh && ( ref $fh eq 'GLOB' ) )
+                 || ( ref $fh && ( UNIVERSAL::can( $fh, 'can' ) 
+                    && ( $fh->isa('IO::Handle') || $fh->isa('IO::String') ) ) ) 
+               ) {
+            $self->throw("file handle $fh doesn't appear to be a handle");
+        }
+    }
+    if ($HAS_EOL) {
+        binmode $fh, ':raw:eol(LF-Native)';
     }
     $self->_fh($fh) if $fh; # if not provided, defaults to STDIN and STDOUT
 
@@ -348,7 +388,7 @@ sub _initialize_io {
 sub _fh {
     my ($obj, $value) = @_;
     if ( defined $value) {
-	$obj->{'_filehandle'} = $value;
+    $obj->{'_filehandle'} = $value;
     }
     return $obj->{'_filehandle'};
 }
@@ -373,7 +413,7 @@ sub _fh {
 
 sub mode {
     my ($obj, @arg) = @_;
-	my %param = @arg;
+    my %param = @arg;
     return $obj->{'_mode'} if defined $obj->{'_mode'} and !$param{-force};
     
     # Previous system of:
@@ -410,13 +450,12 @@ sub mode {
  Returns : value of file
  Args    : newvalue (optional)
 
-
 =cut
 
 sub file {
     my ($obj, $value) = @_;
     if ( defined $value) {
-	$obj->{'_file'} = $value;
+    $obj->{'_file'} = $value;
     }
     return $obj->{'_file'};
 }
@@ -438,6 +477,64 @@ sub _print {
     return $ret;
 }
 
+
+=head2 _insert
+
+    Title   : _insert
+    Usage   : $obj->_insert($string,1)
+    Function: Insert some text in a file at the given line number (1-based).
+    Returns : 1 on success
+    Args    : string to write in file
+              line number to insert the string at
+
+=cut
+
+sub _insert {
+    my ($self, $string, $line_num) = @_;
+    # Line number check
+    if ($line_num < 1) {
+        $self->throw("Cannot insert text at line $line_num because the minimum".
+            " line number possible is 1");
+    }
+    # File check
+    my $file = $self->file;
+    if (not defined $file) {
+        $self->throw('Cannot insert a line in a IO object initialized with ".
+            "anything else than a file.');
+    }
+    $file =~ s/^\+?[><]?//; # transform '+>output.ace' into 'output.ace'
+    # Everything that needs to be written is written before we read it
+    $self->flush;
+    # Edit the file in place, line by line (no slurping)
+    {
+        local @ARGV = ($file);     # input file
+        #local $^I = '~';          # backup file extension, e.g. ~, .bak, .ori
+        local $^I = '';            # no backup file
+        while (<>) {
+            if ($. == $line_num) { # right line for new data
+                print $string.$_;
+            } else {
+                print;
+            }
+        }
+    }
+    # Line number check (again)
+    if ( $. > 0 && $line_num > $. ) {
+        $self->throw("Cannot insert text at line $line_num because there are ".
+            "only $. lines in file $file");
+    }
+    # Re-open the file in append mode to be ready to add text at the end of it
+    # when the next _print() statement comes
+    open my $new_fh, ">>$file" or $self->throw("Cannot append to file $file: $!");
+    $self->_fh($new_fh);
+    # If file is empty and we're inserting at line 1, simply append text to file
+    if ( $. == 0 && $line_num == 1 ) {
+        $self->_print($string);
+    }
+    return 1;
+}
+
+
 =head2 _readline
 
  Title   : _readline
@@ -469,15 +566,15 @@ sub _readline {
     # if the buffer been filled by _pushback then return the buffer
     # contents, rather than read from the filehandle
     if( @{$self->{'_readbuffer'} || [] } ) {
-	$line = shift @{$self->{'_readbuffer'}};
+    $line = shift @{$self->{'_readbuffer'}};
     } else {
-	$line = <$fh>;
+    $line = <$fh>;
     }
-
+    
     #don't strip line endings if -raw is specified
     # $line =~ s/\r\n/\n/g if( (!$param{-raw}) && (defined $line) );
     # Dave Howorth's fix
-    if( (!$param{-raw}) && (defined $line) ) {
+    if( !$HAS_EOL && !$param{-raw} && (defined $line) ) {
         $line =~ s/\015\012/\012/g; # Change all CR/LF pairs to LF
         $line =~ tr/\015/\n/ unless $ONMAC; # Change all single CRs to NEWLINE
     }
@@ -494,11 +591,22 @@ sub _readline {
  Returns : none
  Args    : newvalue
  Note    : This is only supported for pushing back data ending with the
-		   current, localized value of $/. Using this method to push modified
-		   data back onto the buffer stack is not supported; see bug 843.
+           current, localized value of $/. Using this method to push modified
+           data back onto the buffer stack is not supported; see bug 843.
 
 =cut
 
+# fix for bug 843, this reveals some unsupported behavior
+    
+#sub _pushback {
+#    my ($obj, $value) = @_;    
+#    if (index($value, $/) >= 0) {
+#        push @{$obj->{'_readbuffer'}}, $value;
+#    } else {
+#        $obj->throw("Pushing modifed data back not supported: $value");
+#    }
+#}
+
 sub _pushback {
     my ($obj, $value) = @_;
     return unless $value;
@@ -518,17 +626,20 @@ sub _pushback {
 
 sub close {
    my ($self) = @_;
-   return if $self->noclose; # don't close if we explictly asked not to
-   if( defined $self->{'_filehandle'} ) {
+
+   # don't close if we explictly asked not to
+   return if $self->noclose;
+
+   if( defined( my $fh = $self->{'_filehandle'} )) {
        $self->flush;
-       return if( \*STDOUT == $self->_fh ||
-		  \*STDERR == $self->_fh ||
-		  \*STDIN == $self->_fh
-		  ); # don't close STDOUT fh
-       if( ! ref($self->{'_filehandle'}) ||
-	   ! $self->{'_filehandle'}->isa('IO::String') ) {
-	   close($self->{'_filehandle'});
-       }
+       return if     ref $fh eq 'GLOB'
+         && (    \*STDOUT == $fh
+              || \*STDERR == $fh
+              || \*STDIN  == $fh
+                    );
+
+       # don't close IO::Strings
+       close $fh unless ref $fh && $fh->isa('IO::String');
    }
    $self->{'_filehandle'} = undef;
    delete $self->{'_readbuffer'};
@@ -589,24 +700,24 @@ sub _io_cleanup {
 
     # we are planning to cleanup temp files no matter what    
     if( exists($self->{'_rootio_tempfiles'}) &&
-	ref($self->{'_rootio_tempfiles'}) =~ /array/i &&
+    ref($self->{'_rootio_tempfiles'}) =~ /array/i &&
     !$self->save_tempfiles) { 
-	if( $v > 0 ) {
-	    warn( "going to remove files ", 
-		  join(",",  @{$self->{'_rootio_tempfiles'}}), "\n");
-	}
-	unlink  (@{$self->{'_rootio_tempfiles'}} );
+    if( $v > 0 ) {
+        warn( "going to remove files ", 
+          join(",",  @{$self->{'_rootio_tempfiles'}}), "\n");
+    }
+    unlink  (@{$self->{'_rootio_tempfiles'}} );
     }
     # cleanup if we are not using File::Temp
     if( $self->{'_cleanuptempdir'} &&
-	exists($self->{'_rootio_tempdirs'}) &&
-	ref($self->{'_rootio_tempdirs'}) =~ /array/i &&
-    !$self->save_tempfiles) {	
-	if( $v > 0 ) {
-	    warn( "going to remove dirs ", 
-		  join(",",  @{$self->{'_rootio_tempdirs'}}), "\n");
-	}
-	$self->rmtree( $self->{'_rootio_tempdirs'});
+    exists($self->{'_rootio_tempdirs'}) &&
+    ref($self->{'_rootio_tempdirs'}) =~ /array/i &&
+    !$self->save_tempfiles) {   
+    if( $v > 0 ) {
+        warn( "going to remove dirs ", 
+          join(",",  @{$self->{'_rootio_tempdirs'}}), "\n");
+    }
+    $self->rmtree( $self->{'_rootio_tempdirs'});
     }
 }
 
@@ -629,23 +740,22 @@ sub _io_cleanup {
 
 sub exists_exe {
     my ($self, $exe) = @_;
-	$self->throw("Must pass a defined value to exists_exe") unless defined $exe;
+    $self->throw("Must pass a defined value to exists_exe") unless defined $exe;
     $exe = $self if (!(ref($self) || $exe));
     $exe .= '.exe' if(($^O =~ /mswin/i) && ($exe !~ /\.(exe|com|bat|cmd)$/i));
-    return $exe if (-e $exe); # full path and exists
+    return $exe if ( -f $exe && -x $exe ); # full path and exists
 
     # Ewan's comment. I don't think we need this. People should not be
     # asking for a program with a pathseparator starting it
-    
     # $exe =~ s/^$PATHSEP//;
 
     # Not a full path, or does not exist. Let's see whether it's in the path.
     if($FILESPECLOADED) {
-	foreach my $dir (File::Spec->path()) {
-	    my $f = Bio::Root::IO->catfile($dir, $exe);	    
-	    return $f if(-e $f && -x $f );
-	}
-    }    
+        foreach my $dir (File::Spec->path()) {
+            my $f = Bio::Root::IO->catfile($dir, $exe);
+            return $f if( -f $f && -x $f );
+        }
+    }
     return 0;
 }
 
@@ -674,65 +784,65 @@ sub tempfile {
 
     # map between naming with and without dash
     foreach my $key (keys(%params)) {
-	if( $key =~ /^-/  ) {
-	    my $v = $params{$key};
-	    delete $params{$key};
-	    $params{uc(substr($key,1))} = $v;
-	} else { 
-	    # this is to upper case
-	    my $v = $params{$key};
-	    delete $params{$key};	    
-	    $params{uc($key)} = $v;
-	}
+    if( $key =~ /^-/  ) {
+        my $v = $params{$key};
+        delete $params{$key};
+        $params{uc(substr($key,1))} = $v;
+    } else { 
+        # this is to upper case
+        my $v = $params{$key};
+        delete $params{$key};       
+        $params{uc($key)} = $v;
+    }
     }
     $params{'DIR'} = $TEMPDIR if(! exists($params{'DIR'}));
     unless (exists $params{'UNLINK'} && 
-	    defined $params{'UNLINK'} &&
-	    ! $params{'UNLINK'} ) {
-	$params{'UNLINK'} = 1;
+        defined $params{'UNLINK'} &&
+        ! $params{'UNLINK'} ) {
+    $params{'UNLINK'} = 1;
     } else { $params{'UNLINK'} = 0 }
-	    
+        
     if($FILETEMPLOADED) {
-	if(exists($params{'TEMPLATE'})) {
-	    my $template = $params{'TEMPLATE'};
-	    delete $params{'TEMPLATE'};
-	    ($tfh, $file) = File::Temp::tempfile($template, %params);
-	} else {
-	    ($tfh, $file) = File::Temp::tempfile(%params);
-	}
+    if(exists($params{'TEMPLATE'})) {
+        my $template = $params{'TEMPLATE'};
+        delete $params{'TEMPLATE'};
+        ($tfh, $file) = File::Temp::tempfile($template, %params);
     } else {
-	my $dir = $params{'DIR'};
-	$file = $self->catfile($dir,
-			       (exists($params{'TEMPLATE'}) ?
-				$params{'TEMPLATE'} :
-				sprintf( "%s.%s.%s",  
-					 $ENV{USER} || 'unknown', $$, 
-					 $TEMPCOUNTER++)));
+        ($tfh, $file) = File::Temp::tempfile(%params);
+    }
+    } else {
+    my $dir = $params{'DIR'};
+    $file = $self->catfile($dir,
+                   (exists($params{'TEMPLATE'}) ?
+                $params{'TEMPLATE'} :
+                sprintf( "%s.%s.%s",  
+                     $ENV{USER} || 'unknown', $$, 
+                     $TEMPCOUNTER++)));
 
-	# sneakiness for getting around long filenames on Win32?
-	if( $HAS_WIN32 ) {
-	    $file = Win32::GetShortPathName($file);
-	}
+    # sneakiness for getting around long filenames on Win32?
+    if( $HAS_WIN32 ) {
+        $file = Win32::GetShortPathName($file);
+    }
 
-	# Try to make sure this will be marked close-on-exec
-	# XXX: Win32 doesn't respect this, nor the proper fcntl,
-	#      but may have O_NOINHERIT. This may or may not be in Fcntl.
-	local $^F = 2; 
-	# Store callers umask
-	my $umask = umask();
-	# Set a known umaskr
-	umask(066);
-	# Attempt to open the file
-	if ( sysopen($tfh, $file, $OPENFLAGS, 0600) ) {
-	    # Reset umask
-	    umask($umask); 
-	} else { 
-	    $self->throw("Could not open tempfile $file: $!\n");
-	}
+    # Try to make sure this will be marked close-on-exec
+    # XXX: Win32 doesn't respect this, nor the proper fcntl,
+    #      but may have O_NOINHERIT. This may or may not be in Fcntl.
+    local $^F = 2; 
+    # Store callers umask
+    my $umask = umask();
+    # Set a known umaskr
+    umask(066);
+    # Attempt to open the file
+    if ( sysopen($tfh, $file, $OPENFLAGS, 0600) ) {
+        # Reset umask
+        umask($umask); 
+    } else { 
+        $self->throw("Could not open tempfile $file: $!\n");
+    }
     }
 
     if(  $params{'UNLINK'} ) {
-	push @{$self->{'_rootio_tempfiles'}}, $file;
+    push @{$self->{'_rootio_tempfiles'}}, $file;
     } 
 
 
@@ -758,7 +868,7 @@ sub tempfile {
 sub tempdir {
     my ( $self, @args ) = @_;
     if($FILETEMPLOADED && File::Temp->can('tempdir') ) {
-	return File::Temp::tempdir(@args);
+    return File::Temp::tempdir(@args);
     }
 
     # we have to do this ourselves, not good
@@ -766,11 +876,11 @@ sub tempdir {
     # we are planning to cleanup temp files no matter what
     my %params = @args;
     $self->{'_cleanuptempdir'} = ( defined $params{CLEANUP} && 
-				   $params{CLEANUP} == 1);
+                   $params{CLEANUP} == 1);
     my $tdir = $self->catfile($TEMPDIR,
-			      sprintf("dir_%s-%s-%s", 
-				      $ENV{USER} || 'unknown', $$, 
-				      $TEMPCOUNTER++));
+                  sprintf("dir_%s-%s-%s", 
+                      $ENV{USER} || 'unknown', $$, 
+                      $TEMPCOUNTER++));
     mkdir($tdir, 0755);
     push @{$self->{'_rootio_tempdirs'}}, $tdir; 
     return $tdir;
@@ -804,7 +914,7 @@ sub catfile {
     # this is clumsy and not very appealing, but how do we specify the
     # root directory?
     if($args[0] eq '/') {
-	$args[0] = $ROOTDIR;
+    $args[0] = $ROOTDIR;
     }
     return join($PATHSEP, @args);
 }
@@ -847,11 +957,11 @@ sub catfile {
 sub rmtree {
     my($self,$roots, $verbose, $safe) = @_;
     if( $FILEPATHLOADED ) { 
-	return File::Path::rmtree ($roots, $verbose, $safe);
+    return File::Path::rmtree ($roots, $verbose, $safe);
     }
 
     my $force_writeable = ($^O eq 'os2' || $^O eq 'dos' || $^O eq 'MSWin32'
-		       || $^O eq 'amigaos' || $^O eq 'cygwin');
+               || $^O eq 'amigaos' || $^O eq 'cygwin');
     my $Is_VMS = $^O eq 'VMS';
 
     my(@files);
@@ -859,86 +969,86 @@ sub rmtree {
     $verbose ||= 0;
     $safe ||= 0;
     if ( defined($roots) && length($roots) ) {
-	$roots = [$roots] unless ref $roots;
+    $roots = [$roots] unless ref $roots;
     } else {
-	$self->warn("No root path(s) specified\n");
-	return 0;
+    $self->warn("No root path(s) specified\n");
+    return 0;
     }
 
     my($root);
     foreach $root (@{$roots}) {
-	$root =~ s#/\z##;
-	(undef, undef, my $rp) = lstat $root or next;
-	$rp &= 07777;	# don't forget setuid, setgid, sticky bits
-	if ( -d _ ) {
-	    # notabene: 0777 is for making readable in the first place,
-	    # it's also intended to change it to writable in case we have
-	    # to recurse in which case we are better than rm -rf for 
-	    # subtrees with strange permissions
-	    chmod(0777, ($Is_VMS ? VMS::Filespec::fileify($root) : $root))
-	      or $self->warn("Can't make directory $root read+writeable: $!")
-		unless $safe;
-	    if (opendir(DIR, $root) ){
-		@files = readdir DIR;
-		closedir(DIR);
-	    } else {
-	        $self->warn( "Can't read $root: $!");
-		@files = ();
-	    }
+    $root =~ s#/\z##;
+    (undef, undef, my $rp) = lstat $root or next;
+    $rp &= 07777;   # don't forget setuid, setgid, sticky bits
+    if ( -d _ ) {
+        # notabene: 0777 is for making readable in the first place,
+        # it's also intended to change it to writable in case we have
+        # to recurse in which case we are better than rm -rf for 
+        # subtrees with strange permissions
+        chmod(0777, ($Is_VMS ? VMS::Filespec::fileify($root) : $root))
+          or $self->warn("Can't make directory $root read+writeable: $!")
+        unless $safe;
+        if (opendir(DIR, $root) ){
+        @files = readdir DIR;
+        closedir(DIR);
+        } else {
+            $self->warn( "Can't read $root: $!");
+        @files = ();
+        }
 
-	    # Deleting large numbers of files from VMS Files-11 filesystems
-	    # is faster if done in reverse ASCIIbetical order 
-	    @files = reverse @files if $Is_VMS;
-	    ($root = VMS::Filespec::unixify($root)) =~ s#\.dir\z## if $Is_VMS;
-	    @files = map("$root/$_", grep $_!~/^\.{1,2}\z/s,@files);
-	    $count += $self->rmtree([@files],$verbose,$safe);
-	    if ($safe &&
-		($Is_VMS ? !&VMS::Filespec::candelete($root) : !-w $root)) {
-		print "skipped $root\n" if $verbose;
-		next;
-	    }
-	    chmod 0777, $root
-	      or $self->warn( "Can't make directory $root writeable: $!")
-		if $force_writeable;
-	    print "rmdir $root\n" if $verbose;
-	    if (rmdir $root) {
-		++$count;
-	    }
-	    else {
-		$self->warn( "Can't remove directory $root: $!");
-		chmod($rp, ($Is_VMS ? VMS::Filespec::fileify($root) : $root))
-		    or $self->warn("and can't restore permissions to "
-		            . sprintf("0%o",$rp) . "\n");
-	    }
-	}
-	else {
+        # Deleting large numbers of files from VMS Files-11 filesystems
+        # is faster if done in reverse ASCIIbetical order 
+        @files = reverse @files if $Is_VMS;
+        ($root = VMS::Filespec::unixify($root)) =~ s#\.dir\z## if $Is_VMS;
+        @files = map("$root/$_", grep $_!~/^\.{1,2}\z/s,@files);
+        $count += $self->rmtree([@files],$verbose,$safe);
+        if ($safe &&
+        ($Is_VMS ? !&VMS::Filespec::candelete($root) : !-w $root)) {
+        print "skipped $root\n" if $verbose;
+        next;
+        }
+        chmod 0777, $root
+          or $self->warn( "Can't make directory $root writeable: $!")
+        if $force_writeable;
+        print "rmdir $root\n" if $verbose;
+        if (rmdir $root) {
+        ++$count;
+        }
+        else {
+        $self->warn( "Can't remove directory $root: $!");
+        chmod($rp, ($Is_VMS ? VMS::Filespec::fileify($root) : $root))
+            or $self->warn("and can't restore permissions to "
+                    . sprintf("0%o",$rp) . "\n");
+        }
+    }
+    else {
 
-	    if ($safe &&
-		($Is_VMS ? !&VMS::Filespec::candelete($root)
-		         : !(-l $root || -w $root)))
-	    {
-		print "skipped $root\n" if $verbose;
-		next;
-	    }
-	    chmod 0666, $root
-	      or $self->warn( "Can't make file $root writeable: $!")
-		if $force_writeable;
-	    warn "unlink $root\n" if $verbose;
-	    # delete all versions under VMS
-	    for (;;) {
-		unless (unlink $root) {
-		    $self->warn( "Can't unlink file $root: $!");
-		    if ($force_writeable) {
-			chmod $rp, $root
-			    or $self->warn("and can't restore permissions to "
-			            . sprintf("0%o",$rp) . "\n");
-		    }
-		    last;
-		}
-		++$count;
-		last unless $Is_VMS && lstat $root;
-	    }
-	}
+        if ($safe &&
+        ($Is_VMS ? !&VMS::Filespec::candelete($root)
+                 : !(-l $root || -w $root)))
+        {
+        print "skipped $root\n" if $verbose;
+        next;
+        }
+        chmod 0666, $root
+          or $self->warn( "Can't make file $root writeable: $!")
+        if $force_writeable;
+        warn "unlink $root\n" if $verbose;
+        # delete all versions under VMS
+        for (;;) {
+        unless (unlink $root) {
+            $self->warn( "Can't unlink file $root: $!");
+            if ($force_writeable) {
+            chmod $rp, $root
+                or $self->warn("and can't restore permissions to "
+                        . sprintf("0%o",$rp) . "\n");
+            }
+            last;
+        }
+        ++$count;
+        last unless $Is_VMS && lstat $root;
+        }
+    }
     }
 
     $count;
@@ -960,7 +1070,7 @@ sub rmtree {
 sub _flush_on_write {
     my ($self,$value) = @_;
     if( defined $value) {
-	$self->{'_flush_on_write'} = $value;
+    $self->{'_flush_on_write'} = $value;
     }
     return $self->{'_flush_on_write'};
 }
