@@ -1,5 +1,3 @@
-# $Id: chaos.pm 11480 2007-06-14 14:16:21Z sendu $
-# $Date: 2007-06-14 09:16:21 -0500 (Thu, 14 Jun 2007) $
 #
 # BioPerl module for Bio::SeqIO::chaos
 #
@@ -96,13 +94,24 @@ of the Bioperl mailing lists.  Your participation is much appreciated.
   bioperl-l@bioperl.org                  - General discussion
   http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
 
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
+
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution.
 Bug reports can be submitted via the web:
 
-  http://bugzilla.bioperl.org
+  https://redmine.open-bio.org/projects/bioperl/
 
 =head1 AUTHOR - Chris Mungall
 
@@ -168,7 +177,7 @@ sub _initialize {
                                         [export_perl5lib=>$ENV{PERL5LIB}],
                                         [export_program=>$0],
                                         [export_module=>'Bio::SeqIO::chaos'],
-                                        [export_module_cvs_id=>'$Id: chaos.pm 11480 2007-06-14 14:16:21Z sendu $'],
+                                        [export_module_cvs_id=>'$Id$'],
                                        ]);
 
     return;
