@@ -1,4 +1,3 @@
-# $Id: StandAloneNCBIBlast.pm 15231 2008-12-22 21:51:02Z cjfields $
 #
 # BioPerl module for Bio::Tools::Run::StandAloneBlast
 #
@@ -33,13 +32,24 @@ of the Bioperl mailing lists.  Your participation is much appreciated.
   bioperl-l@bioperl.org                  - General discussion
   http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
 
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
+
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution.  Bug reports can be submitted via 
 the web:
 
-  http://bugzilla.open-bio.org/
+  https://redmine.open-bio.org/projects/bioperl/
 
 =head1 AUTHOR - Peter Schattner
 
@@ -63,6 +73,7 @@ methods. Internal methods are usually preceded with a _
 package Bio::Tools::Run::StandAloneNCBIBlast;
 
 use strict;
+use warnings;
 
 use base qw(Bio::Tools::Run::StandAloneBlast);
 
@@ -78,7 +89,7 @@ our %GENERAL_PARAMS  = (i => 'input',
                         o => 'outfile',
                         p => 'program',
                         d => 'database');
-our @BLASTALL_PARAMS = qw(A B C D E F G K L M O P Q R S W X Y Z a b e f l m q r t v w y z);
+our @BLASTALL_PARAMS = qw(A B C D E F G K L M O P Q R S W X Y Z a b e f l m q r t v w y z n);
 our @BLASTALL_SWITCH = qw(I g J T U n V s);
 our @BLASTPGP_PARAMS = qw(A B C E F G H I J K L M N O P Q R S T U W X Y Z a b c e f h j k l m q s t u v y z);
 our @RPSBLAST_PARAMS = qw(F I J L N O P T U V X Y Z a b e l m v y z);
@@ -504,12 +515,12 @@ sub _setparams {
     # workaround for problems with shell metacharacters [bug 2707]
     # simply quoting does not always work!
     my $tmp = $self->o;
-    $self->o(quotemeta($tmp)) if $tmp;
+    $self->o(quotemeta($tmp)) if ($tmp && $^O !~ /^MSWin/);
     
     my $param_string = $self->SUPER::_setparams(-params => [@execparams],
                                                 -dash => 1);
     
-    $self->o($tmp) if $tmp;
+    $self->o($tmp) if ($tmp && $^O !~ /^MSWin/);
 
     $self->d($database) if $database;
     
