@@ -1,4 +1,3 @@
-# $Id: SeqPattern.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # bioperl module for Bio::Tools::SeqPattern
 #
@@ -178,7 +177,7 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution. Bug reports can be submitted via the
 web:
 
-  http://bugzilla.open-bio.org/
+  https://redmine.open-bio.org/projects/bioperl/
 
 =head1 AUTHOR
 
@@ -209,8 +208,6 @@ use strict;
 use vars qw ($ID);
 $ID  = 'Bio::Tools::SeqPattern';
 
-require Bio::Tools::SeqPattern::Backtranslate;
-
 ## These constants may be more appropriate in a Bio::Dictionary.pm
 ## type of class.
 my $PURINES      = 'AG';
@@ -226,7 +223,6 @@ my (%Processed_braces, %Processed_asterics);
 ##                                 CONSTRUCTOR                                     ##
 #####################################################################################
 
-
 =head1 new
 
  Title     : new
@@ -536,6 +532,9 @@ sub revcom {
 
 sub backtranslate {
     my $self = shift;
+    
+    # _load_module loads dynamically, caches call if successful
+    $self->_load_module('Bio::Tools::SeqPattern::Backtranslate');
     Bio::Tools::SeqPattern::Backtranslate->import("_reverse_translate_motif");
 
     if ($self->type ne 'Amino') {
