@@ -1,4 +1,3 @@
-## $Id: dpAlign.pm 16123 2009-09-17 12:57:27Z cjfields $
 
 # BioPerl module for Bio::Tools::dpAlign
 #
@@ -191,7 +190,7 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution. Bug reports can be submitted via the
 web:
 
-  http://bugzilla.open-bio.org/
+  https://redmine.open-bio.org/projects/bioperl/
 
 =head1 AUTHOR
 
@@ -407,7 +406,7 @@ sub pairwise_alignment_score {
 	return Bio::Ext::Align::Score_Protein_Sequences($prof, $seq2->seq);
     }
     else {
-	croak("There is currently no support for the types of sequences you want to align!\n");
+	$self->throw("There is currently no support for the types of sequences you want to align!\n");
 	return;
     }
 }
